#!/usr/bin/python

"""
Script to create the zhsp.txt file.

Takes the input files sprior.npy, obs.txt, bg.txt, calculates
hsprior by convolving sprior with h, then subtracts hsprior and 
background values from bg.txt from the observation values in obs.txt
"""


import sys
import getopt
import numpy

from utils import *

#########################################################################
def usage():

	print """
Usage: make_z.py [--other=otherdata] [-c configfile] obsfile spriorfile backgroundfile

where 

  Reguired arguments
	obsfile            : text file with observation value, one line per receptor.
	spriorfile         : file with sprior data. It can be either .npy or netcdf format.
	backgroundfile     : text file with background value, one line per receptor.

  Optional arguments
	-c configfile      : file with configuration data.  Default is 'config.ini'.
	--other=otherdata  : text file with other values to subtract from observations, one line per receptor.
"""

#########################################################################

configfile = "config.ini"
otherfile = ""

try:
        opts, args = getopt.getopt(sys.argv[1:], "c:", ["other="])
except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        usage()
        sys.exit()

for o, a in opts:
        if o == "-c":
                configfile = a
	elif o == "--other":
		otherfile = a


if len(args) != 3:
        usage()
        sys.exit()


configfile = "config.ini"
config = getConfig(configfile)

obsfile = args[0]
spriorfile = args[1]
backgroundfile = args[2]
# Read in observation values.
try:
        obs = numpy.loadtxt(obsfile)
except:
        sys.exit("Error reading observation file: %s" % obsfile)

#print obs
numobs = obs.shape[0]
if obs.ndim > 1: obs = obs[:,0]


config["num_obs"] = numobs


# -- hsprior
# try reading file as a numpy .npy file first,
# if that doesn't work, try reading it as netcdf file.
print "making hsprior..."
try:
	sprior = numpy.load(spriorfile)
except:
	try:
		landmask = numpy.load(config["landmask_file"])
		sprior = getSpriorNetcdf(spriorfile, "sprior", landmask)
		numpy.save("sprior", sprior)  # save it as a numpy file
        except:
                print "Can not read sprior file."
                sys.exit()


hsp = convolve(sprior, config)

hspriorfile = config["workdir"] + "/hsprior.txt"
print "saving hsprior in ", hspriorfile
numpy.savetxt(hspriorfile, hsp)

if otherfile:
	hsother = numpy.loadtxt(otherfile)


# Get background values previously made using make_bg.py
bg = numpy.loadtxt(backgroundfile)
if bg.ndim > 1: bg = bg[:,0]


# subtract background values from observation
obsbg = obs - bg
bgfile = "obs-bg.txt"
print "Saving obs - bg in %s..." % bgfile
numpy.savetxt(bgfile, obsbg, fmt='%12.3f')


# Subtract hother and hsp from bg
print "making zhsp"
if otherfile:
	zhsp = obsbg - hsother - hsp
else:
	zhsp = obsbg - hsp


zfile = config["workdir"] + "/zhsp.txt"
print "Saving zhsp in %s..." % zfile
numpy.savetxt(zfile, zhsp, fmt="%12.6f")

